//----------------------------------------------------------------------------

//RT-9510S CPU
// overall control

//----------------------------------------------------------------------------

#include "Main.h"
#include "Control.h"
#include "Tuner.h"
#include "Keyboard.h"
#include "Port.h"

//--------------------------- : -------------------------------------

// AM     ,  FM -   .

//       :
const __flash char STEP_A[BANDS]  = { 0,  9,  9,  5,  5, 10 };
//       :
const __flash char STEP_M[BANDS]  = { 0,  1,  9,  5,  5,  5 };
//  :
const __flash int BAND_MIN[BANDS] = { 0, 146,  522,  2200,   5800,   8750 };
//  :
const __flash int BAND_MAX[BANDS] = { 0, 281, 1611,  5825,  15500,  10800 };

#define CHANS      12 // 
#define TUNE_TM  5000 //   , ms
#define BLINK_TM  500 //  , ms
#define SEARCH_TM 100 //   , ms
#define RADIO_TM  400 //  , ms
#define ESIGN  0xBEDA // EEPROM

// :

#define BLANK     0x10
#define MINUS     0x11

//---------------------------- : -----------------------------------

static bool Radio;       //  
static bool Adjust;      // ADJUST (  )
static char Chan;        //  
static char TmrCh;       // ,    
static char Band;        //  
static int  Freq;        //  
static int  BandFmin;    //   
static int  BandFmax;    //   
static char StepM;       //    
static char StepA;       //    
static int  TuneTimer;   //    
static char Show;        // ,     
static bool Memory;      //     
static int  BlinkTimer;  //    
static bool Blink;       //   
static bool Search;      //   
static bool ForceRadio;  //   
static bool PreAS;       //   AS (tune auto stop)
static char SearchTimer; //     
static int  RadioTimer;  //     
__no_init __eeprom int  ESign;        // EEPROM
__no_init __eeprom char EChan[BANDS]; //     EEPROM
__no_init __eeprom int  EFreq[BANDS]; //     EEPROM
__no_init __eeprom int  EChanFreq[BANDS][CHANS + 1]; //   EEPROM
__no_init __eeprom char EForceRadio;  //   

//---------------------------- : -------------------------------

void Control_Init(void)
{
  // EEPROM:
  if(ESign != ESIGN)
  {
    for(char i = 0; i < BANDS; i++)
    {
      EChan[i] = 0;
      EFreq[i] = BAND_MIN[i];
      for(char j = 0; j < CHANS + 1; j++)
        EChanFreq[i][j] = BAND_MIN[i];
      EForceRadio = 0;
      __watchdog_reset();
    }
    ESign = ESIGN;
  }

  Keyboard_Init();   // 
  Port_Init();       //  SPI

  TmrCh = Port_Rx(); //   ( 0,   )

  TuneTimer = 0;
  RadioTimer = 0;
  Radio = 0;
  Band = BAND_NO; // ,   
  Adjust = 0;
  Memory = 0;
  Search = 0;
  Port_MUTE_0;
  ForceRadio = EForceRadio; //  EEPROM
}

//----------------------------------------------------------------------------
//---------------------   : ------------------------
//----------------------------------------------------------------------------

void Control_Exe(bool t)
{
  //  :
  if(t)
  {
    if(TuneTimer) TuneTimer--;
    if(SearchTimer) SearchTimer--;
    if(RadioTimer) RadioTimer--;

    if(BlinkTimer) BlinkTimer--;
    else
    {
      Blink = !Blink;
      BlinkTimer = BLINK_TM;
    }
  }
  // :
  Keyboard_Exe(t);
  //  :
  int key = Keyboard_GetCode();
  int rep = key & ~REP; //    

//------------------- -  : ---------------------

  //  :
  if(Pin_ON && !Radio)
  {
    if(RadioTimer == 0)
    {
      Port_MUTE_1;
      RadioTimer = ms2sys(RADIO_TM);
    }
    if(RadioTimer == 1)
    {
      Radio = 1;
    }
  }
  //  :
  if(Radio && !Pin_ON)
  {
    if(Band != BAND_NO)
    {
      if(EFreq[Band] != Freq) EFreq[Band] = Freq; // 
      if(EChan[Band] != Chan) EChan[Band] = Chan; // 
    }
    Port_Tx(CMD_SHOW_CLOCK);          // 
    Search = 0;
    Port_MUTE_0;
    Radio = 0;
    Band = BAND_NO;
    RadioTimer = 0;
  }

//----------------- -  : -------------------

  //  :
  if((key & KEY_ADJ) && !Adjust)
  {
    Port_Tx(CMD_SHOW_CLOCK);          // 
    Show = CMD_SHOW_CLOCK;
    Memory = 0;
    Search = 0;
    Port_MUTE_0;
    Adjust = 1;
  }
  //  :
  if(Adjust && !(key & KEY_ADJ))
  {
    if(Radio)
    {
      Port_TxRadio(Band, Freq, Chan); // 
      TuneTimer = ms2sys(TUNE_TM);    //   
    }
    else Port_Tx(CMD_SHOW_CLOCK);     // 
    Adjust = 0;
  }
  key &= ~KEY_ADJ; // KEY_ADJ ,    ( 
  rep &= ~KEY_ADJ; // key     .

//------------------   CLOCK-RADIO: ---------------------

  if(Radio && ! Adjust)
  {
    if(key == KEY_DIS)
    {
      ForceRadio = !ForceRadio;
      EForceRadio = ForceRadio;
      TuneTimer = 1;
    }
  }

//----------------------    : -------------------------

  if(Adjust)
  {
    // :
    if(rep == KEY_CH2) Port_Tx(CMD_INC_HRS);
    if(rep == KEY_CH3) Port_Tx(CMD_INC_10M);
    if(rep == KEY_CH4) Port_Tx(CMD_INC_MIN);
    if(rep == KEY_AT)  Port_Tx(CMD_INC_WEEK);
    if((rep == KEY_MEM) && (Show != CMD_SHOW_CLOCK)) Port_Tx(CMD_INC_CHAN);
    //  :
    if(key == KEY_CH6)
    {
      if(Show == CMD_SHOW_CLOCK) Port_Tx(CMD_ADJ_SEC); else
      if(Show == CMD_SHOW_TIMER1)
        { Show = CMD_SHOW_TIMER2; Port_Tx(Show); } else
      if(Show == CMD_SHOW_TIMER2)
        { Show = CMD_SHOW_TIMER1; Port_Tx(Show); }
    }
    if(key == KEY_T_C)
    {
      if(Show == CMD_SHOW_CLOCK) { Show = CMD_SHOW_TIMER1; Port_Tx(Show); }
        else { Show = CMD_SHOW_CLOCK; Port_Tx(Show);  }
    }
    if((key == KEY_CH5) && (Show != CMD_SHOW_CLOCK)) Port_Tx(CMD_CH0);
  }

//------------------------  : --------------------------

  if(Radio)
  {
    char bnd = Keyboard_GetBand();
    if(Band != bnd)
    {
      if(EFreq[Band] != Freq) EFreq[Band] = Freq; // 
      if(EChan[Band] != Chan) EChan[Band] = Chan; // 

      Band = bnd;
      Search = 0;
      Port_MUTE_0;

      BandFmax = BAND_MAX[Band];
      BandFmin = BAND_MIN[Band];
      StepM = STEP_M[Band];
      StepA = STEP_A[Band];

      //  :
      if(TmrCh) { Chan = TmrCh; EChan[Band] = TmrCh; TmrCh = 0; }
      //   EEPROM:
        else Chan = EChan[Band];
      //   ,  :
      if(Chan) Freq = EChanFreq[Band][Chan];
      //    EEPROM:
        else Freq = EFreq[Band];

      Tuner_Load(Band, Freq);           // 
      if(!Adjust)
      {
        Port_TxRadio(Band, Freq, Chan); // 
        TuneTimer = ms2sys(TUNE_TM);    //   
      }
    }
  }

//------------------------ - : -------------------------

  if(Radio && !Adjust)
  {
    //    :
    if(key == KEY_MEM)
    {
      Port_TxRadio(Band, Freq, Chan); // 
      TuneTimer = ms2sys(TUNE_TM);    //   
      BlinkTimer = 0;
      Blink = 0;
      Search = 0;
      Port_MUTE_0;
      Memory = 1;
    }
    //  :
    char ch = 0;
    if(key == KEY_CH1) ch = 1;
    if(key == KEY_CH2) ch = 2;
    if(key == KEY_CH3) ch = 3;
    if(key == KEY_CH4) ch = 4;
    if(key == KEY_CH5) ch = 5;
    if(key == KEY_CH6) ch = 6;
    if(key == (KEY_CH1 | KEY_FAST)) ch = 7;
    if(key == (KEY_CH2 | KEY_FAST)) ch = 8;
    if(key == (KEY_CH3 | KEY_FAST)) ch = 9;
    if(key == (KEY_CH4 | KEY_FAST)) ch = 10;
    if(key == (KEY_CH5 | KEY_FAST)) ch = 11;
    if(key == (KEY_CH6 | KEY_FAST)) ch = 12;
    if(ch)
    {
      Chan = ch;
      Search = 0;
      Port_MUTE_0;
      if(Memory)
      {
        // :
        EChanFreq[Band][Chan] = Freq;
        EChan[Band] = Chan;
        Port_TxRadio(Band, Freq, Chan); // 
        TuneTimer = ms2sys(TUNE_TM);    //   
        Memory = 0;
      }
      else
      {
        // :
        Freq = EChanFreq[Band][Chan];
        EChan[Band] = Chan;
        Tuner_Load(Band, Freq);         // 
        Port_TxRadio(Band, Freq, Chan); // 
        TuneTimer = ms2sys(TUNE_TM);    //   
      }
    }
    //  :
    if(Memory)
    {
      if(Blink) Port_TxRadio(Band, Freq, MINUS);
        else Port_TxRadio(Band, Freq, 0);
    }
  }

//----------------------------  : -----------------------------

  if(Radio && !Adjust)
  {
    int f = Freq;
    // :
    if(rep == KEY_TUP) f = f + StepM;
    if(rep == KEY_TDN) f = f - StepM;
    if(rep == (KEY_TUP | KEY_FAST)) f = f + StepM * 10;
    if(rep == (KEY_TDN | KEY_FAST)) f = f - StepM * 10;
    if(f > BandFmax) f =  BandFmax;
    if(f < BandFmin) f =  BandFmin;
    if(f != Freq)
    {
      Freq = f;
      Chan = 0;
      Search = 0;
      Port_MUTE_0;
      Tuner_Load(Band, Freq);         // 
      Port_TxRadio(Band, Freq, Chan); //  LCD
      TuneTimer = ms2sys(TUNE_TM);    //   
    }
  }

//------------------------  : -------------------------

  if(Radio && !Adjust)
  {
    //  :
    if(key == KEY_AT)
    {
      if(Freq == BandFmax) Freq = BandFmin;
      Search = 1;
      Port_MUTE_1;
      PreAS = Pin_AS;
      SearchTimer = 0;
    }
    //  :
    if(Search && !SearchTimer)
    {
      if(!PreAS && Pin_AS)
      {
        Search = 0;
        Port_MUTE_0;
      }
      else
      {
        PreAS = Pin_AS;
        Chan = 0;
        Freq = Freq - Freq % StepA + StepA; //   
        if(Freq >= BandFmax)             //
        {
          Freq = BandFmax;
          Search = 0;
          Port_MUTE_0;
        }
        Tuner_Load(Band, Freq);          // 
        Port_TxRadio(Band, Freq, Chan);  // 
        TuneTimer = ms2sys(TUNE_TM);     //   
        SearchTimer = ms2sys(SEARCH_TM); //  
      }
    }
  }

//---------------    RADIO-CLOCK: -------------------

  if(Radio && (TuneTimer == 1))
  {
    Memory = 0;
    if(EFreq[Band] != Freq) EFreq[Band] = Freq; // 
    if(EChan[Band] != Chan) EChan[Band] = Chan; // 
    if(!Adjust)
    {
      if(!ForceRadio) Port_Tx(CMD_SHOW_CLOCK);  // 
        else Port_TxRadio(Band, Freq, Chan);    // 
    }
  }

}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
