//----------------------------------------------------------------------------

//RT-9510S CPU
//  

//----------------------------------------------------------------------------

#include "Main.h"
#include "Keyboard.h"
#include "Tuner.h"

//--------------------------- : -------------------------------------

#define DB_TM      50 //  , ms
#define NOREP_TM 1000 //  , ms
#define REP_TM    150 // , ms

//---------------------------- : -----------------------------------

static char BandCode;
static int KeyCode;

//--------------------------  : ------------------------------

long Scan(void); // 

//------------------------  : -------------------------

void Keyboard_Init(void)
{
  BandCode = 0;
  KeyCode = 0;
}

//-----------------------  : ---------------------------

long Scan(void)
{
  char d; long s = 0;
  for(char i = 0; i < 5; i++)
  {
    d = 0;
    switch(i)                      //   -
    {
    case 0: Port_SCAN0_1; break;
    case 1: Port_SCAN1_1; break;
    case 2: Port_SCAN2_1; break;
    case 3: Port_SCAN3_1; break;
    case 4: Port_SCAN4_1; break;
    }
    Delay_us(10);                  //  
    if(Pin_RET0) d = d | (0x01);   //  
    if(Pin_RET1) d = d | (0x02);
    if(Pin_RET2) d = d | (0x04);
    if(Pin_RET3) d = d | (0x08);
    Port_SCAN0_0;                  //  -
    Port_SCAN1_0;
    Port_SCAN2_0;
    Port_SCAN3_0;
    Port_SCAN4_0;
    s = s | ((unsigned long)d << (i * 4));
  }
  return(s);
}

//-------------------------  : ----------------------------

void Keyboard_Exe(bool t)
{
  static int LastCode = 0;
  static int TempCode = 0;
  static char TempBand = 0;
  static char BndTimer = 0;
  static char DbncTimer = 0;
  static int RepTimer = 0;

  if(t)
  {
    if(BndTimer) BndTimer--;
    if(DbncTimer) DbncTimer--;
    if(RepTimer) RepTimer--;

    long s = Scan();           // 

    char b = (s >> 15) & 0x1F; //  
    if(b != BandCode)          // 
    {
      if(b != TempBand)        //
      {
        TempBand = b;
        BndTimer = ms2sys(DB_TM);
      }
      else
      {
        if(!BndTimer)
          BandCode = b;        //  
      }
    }

    int  k = s & 0x7FFF;       //  
    if(k != LastCode)          // 
    {
      if(k != TempCode)        //
      {
        TempCode = k;
        DbncTimer = ms2sys(DB_TM);
      }
      else
      {
        if(!DbncTimer)
        {
          if(k != 0)           // 
          {
            RepTimer = ms2sys(NOREP_TM);
            KeyCode = k;
          }
          else KeyCode = 0; // 
          LastCode = k;
        }
      }
    }
    else //
    {
      if(RepTimer == 1)
      {
        if(k & KEY_REP)
        {
          KeyCode = k | REP;
          RepTimer = ms2sys(REP_TM);
        }
      }
    }
  }
}

//---------------------------  : ------------------------------

char Keyboard_GetBand(void)
{
  char bnd = BAND_NO;
  if(BandCode & DET_FM)  bnd = BAND_FM;  else
  if(BandCode & DET_SW2) bnd = BAND_SW2; else
  if(BandCode & DET_SW1) bnd = BAND_SW1; else
  if(BandCode & DET_MW)  bnd = BAND_MW;  else
  if(BandCode & DET_LW)  bnd = BAND_LW;
  return(bnd);
}

//--------------------------   : -----------------------------

int Keyboard_GetCode(void)
{
  int k = KeyCode;
  KeyCode &= KEY_MOD;
  return(k);
}

//----------------------------------------------------------------------------
