//----------------------------------------------------------------------------

//RT-9510S CPU
// 

//Project:  RT-9510S CPU
//Compiler: IAR EWAVR 6.12
//Target:   ATmega88
//Version:  1.0

//Fuses:
//EXTENDED = 0xF9
//HIGH = 0xD4
//LOW = 0xE2

//  RC-  8 .

//     Timer0   CTC.
//    OCR0A.     T_SYS
//  fTick.     Main_GetTick()
//      ,   
//   .

//----------------------------------------------------------------------------

#include "Main.h"
#include "Control.h"

#include "Port.h"

//----------------------------- : ----------------------------------

volatile bool fTick;               //   
static bool tick;                  //    

//--------------------------  : ------------------------------

void main(void);                   // 
void Main_Wdt_Init(void);          // watchdog-
void Main_Rst_Wdt(bool t);         // watchdog-
void Main_Ports_Init(void);        // 
void Main_Timer_Init(void);        //  
bool Main_GetTick(void);           //  
#pragma vector = TIMER0_COMPA_vect
__interrupt void Main_Timer(void); //  

//----------------------------------------------------------------------------
//--------------------------  : -----------------------------
//----------------------------------------------------------------------------

void main(void)
{
  Main_Wdt_Init();                 // watchdog 
  Main_Ports_Init();               // 
  Main_Timer_Init();               //  
  Control_Init();                  //  
  __enable_interrupt();            // 

  while(1)                         // 
  {
    tick = Main_GetTick();         //  
    Control_Exe(tick);             //   
    Main_Rst_Wdt(tick);            // watchdog-
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//---------------------  watchdog-: ----------------------

void Main_Wdt_Init(void)
{
  __watchdog_reset();
  WDTCSR |= (1 << WDCE) | (1 << WDE); // watchdog-
  WDTCSR = (1 << WDE) | (1 << WDP2) | (1 << WDP0); // 500 
}

//------------------------  watchdog-: -------------------------

void Main_Rst_Wdt(bool t)
{
  if(t)                            //   ,
    __watchdog_reset();            // watchdog-
}

//-------------------------  : ----------------------------

void Main_Ports_Init(void)
{
  DDRB  = I_DDRB;                  //  B
  PORTB = I_PORTB;
  DDRC  = I_DDRC;                  //  C
  PORTC = I_PORTC;
  DDRD  = I_DDRD;                  //  D
  PORTD = I_PORTD;
  ACSR = (1 << ACD);               //  
}

//------------------   : -----------------------

#define TC0_VAL ((int)((F_CLK * T_SYS / DIV_SYS) + 0.5))

void Main_Timer_Init(void)
{
  TCCR0A = (1 << WGM01);           //CTC
  TCCR0B = (1 << CS01) | (1 << CS00); //CK/64
  OCR0A = TC0_VAL - 1;             // 
  TIFR0 = (1 << OCF0A);            //  
  TIMSK0 = (1 << OCIE0A);          //   
  fTick = 1;                       //   
}

//---------------------   : -------------------------

__monitor bool Main_GetTick(void)
{
  if(!fTick) return(0);            //   
  fTick = 0; return(1);            // ,   
}

//-------------------   : -------------------------

#pragma vector = TIMER0_COMPA_vect
__interrupt void Sys_Timer(void)
{
  fTick = 1;                       //   
}

//----------------------------------------------------------------------------
