//----------------------------------------------------------------------------

//RT-9510S CPU
//    TC9125BP + TD6102P

//----------------------------------------------------------------------------

#include "Main.h"
#include "Tuner.h"

//--------------------------- : -------------------------------------

#define ADDR_1     11 //   N
#define ADDR_10    12 //   N
#define ADDR_100   13 //   N
#define ADDR_1000  14 //   N
#define ADDR_MODE  15 //  MODE

#define STEP_1K     0 // 1 
#define STEP_5K     1 // 5 
#define STEP_9K     2 // 9 
#define STEP_10K    3 // 10 
#define STEP_12K5   4 // 12.5 
#define STEP_12K5S  5 // 12.5  + shift 50 
#define STEP_15K625 7 // 15.625 

#define IF_AM     450 //AM IF frequency, kHz
#define IF_FM    1070 //FM IF frequency, x10 kHz
#define PRESCALER   8 //FM prescaler ratio

//--------------------------  : ------------------------------

void Tuner_Tx(char d);

//-------------------------  : ----------------------------

//b = band code (1..5)
//f = frequency AM [kHz], FM [x10 kHz]

void Tuner_Load(char b, int f)
{
  int n = 0;
  char s = 0;
  switch(b)
  {
  case BAND_LW:  s = STEP_1K; n =  f + IF_AM;       break;
  case BAND_MW:  s = STEP_9K; n = (f + IF_AM) / 9;  break;
  case BAND_SW1:
  case BAND_SW2: s = STEP_5K; n = (f + IF_AM) / 5;  break;
  case BAND_FM:  s = ((f % 10)? STEP_12K5S : STEP_12K5);
                 n = (long)(f + IF_FM) * 100 / (125 * PRESCALER); break;
  }
  Tuner_Tx(ADDR_MODE);
  Tuner_Tx(s);
  Tuner_Tx(ADDR_1000);
  Tuner_Tx(n / 1000); n = n % 1000;
  Tuner_Tx(ADDR_100);
  Tuner_Tx(n / 100); n = n % 100;
  Tuner_Tx(ADDR_10);
  Tuner_Tx(n / 10); n = n % 10;
  Tuner_Tx(ADDR_1);
  Tuner_Tx(n);
}

//---------------------    : ------------------------

void Tuner_Tx(char d)
{
  (d & 1)? Port_PLLA_1 : Port_PLLA_0;
  (d & 2)? Port_PLLB_1 : Port_PLLB_0;
  (d & 4)? Port_PLLC_1 : Port_PLLC_0;
  (d & 8)? Port_PLLD_1 : Port_PLLD_0;
  Delay_us(20);
  Port_LOAD_1;
  Delay_us(10);
  Port_LOAD_0;
  Delay_us(20);
  Port_PLLA_0;
  Port_PLLB_0;
  Port_PLLC_0;
  Port_PLLD_0;
}

//----------------------------------------------------------------------------
