//----------------------------------------------------------------------------

//RT-9510S LCD
//  LCD-

//----------------------------------------------------------------------------

// LCD ,   3   
//24   ( 72 ).

//----------------------------------------------------------------------------

#include "Main.h"
#include "Display.h"

//----------------------------- : -----------------------------------

#define LCD_SEG_COUNT 72 //  LCD
#define LCD_BUFF_SIZE (LCD_SEG_COUNT / 8) //  LCD

// :

//SCAN0 (LCDDR00, LCD_Buff[0]):
#define SEG_MW    0x01
#define SEG_LW    0x02
#define SEG_E1    0x04
#define SEG_D1    0x08
#define SEG_SUN   0x10
#define SEG_MON   0x20
#define SEG_D2    0x40
#define SEG_C2    0x80

//SCAN0 (LCDDR01, LCD_Buff[1]):
#define SEG_TUE   0x01
#define SEG_WED   0x02
#define SEG_D3    0x04
#define SEG_C3    0x08
#define SEG_P     0x10
#define SEG_D4    0x20
#define SEG_THU   0x40
#define SEG_FRI   0x80

//SCAN0 (LCDDR02, LCD_Buff[2]):
#define SEG_D5    0x01
#define SEG_C5    0x02
#define SEG_SAT   0x04
#define SEG_D6    0x08
#define SEG_C6    0x10
#define SEG_CH    0x20
#define SEG_D7    0x40
#define SEG_EVE   0x80

//SCAN1 (LCDDR05, LCD_Buff[3]):
#define SEG_FM    0x01
#define SEG_PM    0x02
#define SEG_F1    0x04
#define SEG_G1    0x08
#define SEG_C1    0x10
#define SEG_E2    0x20
#define SEG_G2    0x40
#define SEG_B2    0x80

//SCAN1 (LCDDR06, LCD_Buff[4]):
#define SEG_DP    0x01
#define SEG_E3    0x02
#define SEG_G3    0x04
#define SEG_B3    0x08
#define SEG_E4    0x10
#define SEG_G4    0x20
#define SEG_C4    0x40
#define SEG_E5    0x80

//SCAN1 (LCDDR07, LCD_Buff[5]):
#define SEG_G5    0x01
#define SEG_B5    0x02
#define SEG_E6    0x04
#define SEG_G6    0x08
#define SEG_B6    0x10
#define SEG_E7    0x20
#define SEG_G7    0x40
#define SEG_C7    0x80

//SCAN2 (LCDDR10, LCD_Buff[6]):
#define SEG_SW    0x01
#define SEG_AM    0x02
#define SEG_CLOCK 0x04
#define SEG_A1    0x08
#define SEG_B1    0x10
#define SEG_F2    0x20
#define SEG_A2    0x40
#define SEG_RADIO 0x80

//SCAN2 (LCDDR11, LCD_Buff[7]):
#define SEG_PP    0x01
#define SEG_F3    0x02
#define SEG_A3    0x04
#define SEG_TIMER 0x08
#define SEG_F4    0x10
#define SEG_A4    0x20
#define SEG_B4    0x40
#define SEG_F5    0x80

//SCAN2 (LCDDR12, LCD_Buff[8]):
#define SEG_A5    0x01
#define SEG_KHZ   0x02
#define SEG_F6    0x04
#define SEG_A6    0x08
#define SEG_MHZ   0x10
#define SEG_F7    0x20
#define SEG_A7    0x40
#define SEG_B7    0x80

// :
enum
{
  A = 0x01,
  B = 0x02,
  C = 0x04,
  D = 0x08,
  E = 0x10,
  F = 0x20,
  G = 0x40
};

// :
#define BLANK     0x10
#define MINUS     0x11

//----------------------------- : ----------------------------------

char LCD_Buff[LCD_BUFF_SIZE]; //LCD display buffer (for double buffering)
char Buffer[5];               //buffer   BCD-

//--------------------------  : ------------------------------

char Conv(char d);               //    7-seg 
void LCD_Clear(void);            //  LCD
void LCD_Dig(char n, char d);    // 
void LCD_Update(void);           //    
void Int2BCD(int x, char *buff); //BCD-
void LCD_ShowFM(int f);          //    FM
void LCD_ShowLWMW(int f);        //    LW  MW
void LCD_ShowSW(int f);          //    SW

//-------------------------  : ---------------------------

void LCD_Init(void)
{
  LCD_Clear();	       //clear segment buffer
  LCDCRA = (1<<LCDEN); //enable LCD | (1<<LCDAB);  //Low Power Waveform
  //575 us drive time, max contrast:
  LCDCCR = (1<<LCDDC2) | (1<<LCDCC3) | (1<<LCDCC2) | (1<<LCDCC1) | (1<<LCDCC0);
  //asynchronous clock, 3 COMs, 24 SEGs:
  LCDCRB = (1<<LCDCS) | (1<<LCDMUX1) | (1<<LCDPM2) | (1<<LCDPM1);
  //LCD prescaler 32768 / 16 (N) / 5 (D) / 6 (K) = 68 Hz refresh:
  LCDFRR = (1<<LCDCD2);
}

//-------------------------   LCD: ------------------------------

void LCD_Clear(void)
{
  for(char i = 0; i < LCD_BUFF_SIZE; i++)
    LCD_Buff[i] = 0;
}

//------------     7- : ---------------

//   :
//
//    -- A --
//   |       |
//   F       B
//   |       |
//    -- G --
//   |       |
//   E       C
//   |       |
//    -- D --

char Conv(char d)
{
  // :
  static char __flash Font[]=
  {
    A | B | C | D | E | F    , // 00H - "0"
    B | C                    , // 01H - "1"
    A | B | D | E | G        , // 02H - "2"
    A | B | C | D | G        , // 03H - "3"
    B | C | F | G            , // 04H - "4"
    A | C | D | F | G        , // 05H - "5"
    A | C | D | E | F | G    , // 06H - "6"
    A | B | C                , // 07H - "7"
    A | B | C | D | E | F | G, // 08H - "8"
    A | B | C | D | F | G    , // 09H - "9"
    A | B | C | E | F | G    , // 0AH - "A"
    C | D | E | F | G        , // 0BH - "b"
    A | D | E | F            , // 0CH - "C"
    B | C | D | E | G        , // 0DH - "d"
    A | D | E | F | G        , // 0EH - "E"
    A | E | F | G            , // 0FH - "F"
    0                        , // 10H - " "
    G                          // 11H - "-"
  };

  if(d > MINUS) d = MINUS; //,   
  return(Font[d]); // 7- 
}

//----------------------------    LCD: ---------------------------

//n -   (1..4 -  , 5..6 - , 7 -  )
//d -   0  9,   A  F, 0x10 - , 0x11   - 

void LCD_Dig(char n, char d)
{
  char s = Conv(d);
  switch(n)
  {
  case 1:
    if(s & A) LCD_Buff[6] |= SEG_A1;
    if(s & B) LCD_Buff[6] |= SEG_B1;
    if(s & C) LCD_Buff[3] |= SEG_C1;
    if(s & D) LCD_Buff[0] |= SEG_D1;
    if(s & E) LCD_Buff[0] |= SEG_E1;
    if(s & F) LCD_Buff[3] |= SEG_F1;
    if(s & G) LCD_Buff[3] |= SEG_G1;
    break;
  case 2:
    if(s & A) LCD_Buff[6] |= SEG_A2;
    if(s & B) LCD_Buff[3] |= SEG_B2;
    if(s & C) LCD_Buff[0] |= SEG_C2;
    if(s & D) LCD_Buff[0] |= SEG_D2;
    if(s & E) LCD_Buff[3] |= SEG_E2;
    if(s & F) LCD_Buff[6] |= SEG_F2;
    if(s & G) LCD_Buff[3] |= SEG_G2;
    break;
  case 3:
    if(s & A) LCD_Buff[7] |= SEG_A3;
    if(s & B) LCD_Buff[4] |= SEG_B3;
    if(s & C) LCD_Buff[1] |= SEG_C3;
    if(s & D) LCD_Buff[1] |= SEG_D3;
    if(s & E) LCD_Buff[4] |= SEG_E3;
    if(s & F) LCD_Buff[7] |= SEG_F3;
    if(s & G) LCD_Buff[4] |= SEG_G3;
    break;
  case 4:
    if(s & A) LCD_Buff[7] |= SEG_A4;
    if(s & B) LCD_Buff[7] |= SEG_B4;
    if(s & C) LCD_Buff[4] |= SEG_C4;
    if(s & D) LCD_Buff[1] |= SEG_D4;
    if(s & E) LCD_Buff[4] |= SEG_E4;
    if(s & F) LCD_Buff[7] |= SEG_F4;
    if(s & G) LCD_Buff[4] |= SEG_G4;
    break;
  case 5:
    if(s & A) LCD_Buff[8] |= SEG_A5;
    if(s & B) LCD_Buff[5] |= SEG_B5;
    if(s & C) LCD_Buff[2] |= SEG_C5;
    if(s & D) LCD_Buff[2] |= SEG_D5;
    if(s & E) LCD_Buff[4] |= SEG_E5;
    if(s & F) LCD_Buff[7] |= SEG_F5;
    if(s & G) LCD_Buff[5] |= SEG_G5;
    break;
  case 6:
    if(s & A) LCD_Buff[8] |= SEG_A6;
    if(s & B) LCD_Buff[5] |= SEG_B6;
    if(s & C) LCD_Buff[2] |= SEG_C6;
    if(s & D) LCD_Buff[2] |= SEG_D6;
    if(s & E) LCD_Buff[5] |= SEG_E6;
    if(s & F) LCD_Buff[8] |= SEG_F6;
    if(s & G) LCD_Buff[5] |= SEG_G6;
    break;
  case 7:
    if(s & A) LCD_Buff[8] |= SEG_A7;
    if(s & B) LCD_Buff[8] |= SEG_B7;
    if(s & C) LCD_Buff[5] |= SEG_C7;
    if(s & D) LCD_Buff[2] |= SEG_D7;
    if(s & E) LCD_Buff[5] |= SEG_E7;
    if(s & F) LCD_Buff[8] |= SEG_F7;
    if(s & G) LCD_Buff[5] |= SEG_G7;
    break;
  }
}

//-----------------     : ---------------------

void LCD_Update(void)
{
  LCDDR00 = LCD_Buff[0];
  LCDDR01 = LCD_Buff[1];
  LCDDR02 = LCD_Buff[2];
  LCDDR05 = LCD_Buff[3];
  LCDDR06 = LCD_Buff[4];
  LCDDR07 = LCD_Buff[5];
  LCDDR10 = LCD_Buff[6];
  LCDDR11 = LCD_Buff[7];
  LCDDR12 = LCD_Buff[8];
}

//-----------------------  Int2BCD: ----------------------------

//    -:
//x -    (16 ,  )
//buff -   (5 )

void Int2BCD(int x, char *buff)
{
  for(char i = 0; i < 5; i++)
    buff[i] = 0;                      //  
  if(x < 0) x = -x;
  for(char i = 0; i < 16; i++)        //    
  {
    char c = (x >> 15) & 1;           // 
    x = x << 1;                       //  
    for(char p = 0; p < 5; p++)       //   
    {
      char s = buff[p];               // 
      s = (s << 1) | c; c = 0;        //   
      if(s > 9) { s += 0x06; c = 1; } // 
      s &= 0x0F;                      // 
      buff[p] = s;                    // 
    }
  }
}

//---------------------------  : ---------------------------------

void LCD_Time(time *tm)
{
  LCD_Dig(1, tm->hrs / 10);   // 
  LCD_Dig(2, tm->hrs % 10);
  LCD_Dig(3, tm->min / 10);   // 
  LCD_Dig(4, tm->min % 10);
  switch(tm->day)
  {
  case 0: LCD_Buff[2] |= SEG_EVE; break;
  case 1: LCD_Buff[0] |= SEG_SUN; break;
  case 2: LCD_Buff[0] |= SEG_MON; break;
  case 3: LCD_Buff[1] |= SEG_TUE; break;
  case 4: LCD_Buff[1] |= SEG_WED; break;
  case 5: LCD_Buff[1] |= SEG_THU; break;
  case 6: LCD_Buff[1] |= SEG_FRI; break;
  case 7: LCD_Buff[2] |= SEG_SAT; break;
  }
  LCD_Buff[7] |= SEG_PP;
}

//---------------------------   FM: ------------------------------

//f -   8750..10800 (87.5..108.00 MHz)

void LCD_ShowFM(int f)
{
  Int2BCD(f, Buffer);
  if(Buffer[4]) LCD_Dig(1, Buffer[4]);
  LCD_Dig(2, Buffer[3]);
  LCD_Dig(3, Buffer[2]);
  LCD_Dig(4, Buffer[1]);
  LCD_Dig(5, Buffer[0]);
  LCD_Buff[8] |= SEG_MHZ;
  LCD_Buff[1] |= SEG_P;
}

//-------------------------   LW, MW: ----------------------------

//f -   146..1611 kHz

void LCD_ShowLWMW(int f)
{
  Int2BCD(f, Buffer);
  if(Buffer[3]) LCD_Dig(1, Buffer[3]);
  LCD_Dig(2, Buffer[2]);
  LCD_Dig(3, Buffer[1]);
  LCD_Dig(4, Buffer[0]);
  LCD_Buff[8] |= SEG_KHZ;
}

//---------------------------   SW: ------------------------------

//f -   2200..15500 (2.200..15.500 MHz)

void LCD_ShowSW(int f)
{
  Int2BCD(f, Buffer);
  if(Buffer[4]) LCD_Dig(1, Buffer[4]);
  LCD_Dig(2, Buffer[3]);
  LCD_Dig(3, Buffer[2]);
  LCD_Dig(4, Buffer[1]);
  LCD_Dig(5, Buffer[0]);
  LCD_Buff[8] |= SEG_MHZ;
  LCD_Buff[4] |= SEG_DP;
}

//------------------    : ----------------------

void LCD_ShowRadio(char b, int f, char ch)
{
  LCD_Clear();
  LCD_Buff[6] |= SEG_RADIO;
  switch(b)
  {
  case BAND_FM:
    LCD_Buff[3] |= SEG_FM;
    LCD_ShowFM(f);
    break;
  case BAND_LW:
    LCD_Buff[0] |= SEG_LW;
    LCD_ShowLWMW(f);
    break;
  case BAND_MW:
    LCD_Buff[0] |= SEG_MW;
    LCD_ShowLWMW(f);
    break;
  case BAND_SW1:
  case BAND_SW2:
    LCD_Buff[6] |= SEG_SW;
    LCD_ShowSW(f);
    break;
  }
  if(ch)
  {
    LCD_Buff[2] |= SEG_CH;
    LCD_Dig(7, ch);
  }
  LCD_Update();
}

//----------------------------  : ----------------------------------

void LCD_ShowClock(time *tm)
{
  LCD_Clear();
  LCD_Time(tm);
  LCD_Dig(5, tm->sec / 10);   // 
  LCD_Dig(6, tm->sec % 10);
  LCD_Buff[6] |= SEG_CLOCK;
  LCD_Update();
}

//---------------------------  : ---------------------------------

void LCD_ShowTimer(char n, time *tm, char ch)
{
  LCD_Clear();
  LCD_Time(tm);
  LCD_Dig(6, n); //  
  LCD_Buff[7] |= SEG_TIMER;
  LCD_Buff[2] |= SEG_CH;
  LCD_Dig(7, ch? ch : MINUS);
  LCD_Update();
}

//--------------------     SPI: --------------------------

void LCD_PrintPort(void)
{
  if(SPSR & (1<<SPIF))
  {
    char x = SPDR;
    LCD_Clear();
    LCD_Dig(1, (x >> 4) & 0x0F);
    LCD_Dig(2, x & 0x0F);
    LCD_Update();
  }
}

//----------------------------------------------------------------------------
