//----------------------------------------------------------------------------

//Project:  RT-9510S LCD -  
//Compiler: IAR EWAVR 6.12
//Target:   ATmega329V
//Version:  1.0

//Fuses (BOD off, CKDIV8 off, Int RC 6CK + 4.1ms):
//EXTENDED = 0xFF
//HIGH = 0x99
//LOW = 0xD2

//  ,      
//Power Save.       Timer2, 
//     32768 ,
//  LCD.      
//  .     
//  RC-  8 .  
//       .
//    .      
//    200 .    
//  35 .

//   ,   
//   POWER ON.      
//  RC-.      
//      .  
//     , 
//   .

//       
//    TIMER SET ON  
//         
// .  60    .
//   60     ,
//    TIMER SET ON  SLEEP.

//   ,     
//      SCK.  
//       LCD.

//      SPDR    .
// -    CPU  SCK  
//L-H,       SCK  LCD
//  H-L,   SPDR   .

//----------------------------------------------------------------------------

#include "Main.h"
#include "RTC.h"
#include "Display.h"
#include "Port.h"

//----------------------------- : -----------------------------------

#define SAMPLES 200 //   CLK  RX

//----------------------------- : ----------------------------------

static char SampleCount; //   CLK  RX
static bool OnState;     //  
static bool ExtPower;    //  
static bool OffEn;       // 

//--------------------------  : ------------------------------

void main(void);             // 
void Main_Ports_Init(void);  // 

//----------------------------------------------------------------------------
//--------------------------  : -----------------------------
//----------------------------------------------------------------------------

void main(void)
{
  Main_Ports_Init();         // 
  LCD_Init();                // LCD
  RTC_Init();                // RTC
  Port_Init();               //  SPI-
  OnState = 1;
  Main_PowerOff();           //  

  __enable_interrupt();      // 

  while(1)                   // 
  {
    if(OnState)
    {
      Port_Exe(); //   
      if(OffEn)
      {
        if(SampleCount == SAMPLES)
        {
          if(!ExtPower) Main_PowerOff();  // RX = CLK = 1,  
          if(Pin_SLEEP) RTC_SetOffTime(); //  SLEEP.
          SampleCount = 0;
          ExtPower = 0;
        }
        if(!Pin_SCK) ExtPower = 1;
        SampleCount++;
      }
    }
    else
    {
      __sleep();
    }
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//-------------------------  : ----------------------------

void Main_Ports_Init(void)
{
  DDRA  = I_DDRA;                  //  A
  PORTA = I_PORTA;
  DDRB  = I_DDRB;                  //  B
  PORTB = I_PORTB;
  DDRC  = I_DDRC;                  //  C
  PORTC = I_PORTC;
  DDRD  = I_DDRD;                  //  D
  PORTD = I_PORTD;
  DDRE  = I_DDRE;                  //  E
  PORTE = I_PORTE;
  DDRF  = I_DDRF;                  //  F
  PORTF = I_PORTF;
  DDRG  = I_DDRG;                  //  G
  PORTG = I_PORTG;
  ACSR = (1 << ACD);               //  
}

//------------------------------- : ---------------------------------

__monitor void Main_PowerOn(char ch)
{
  if(!OnState)
  {
    Port_TMROUT_1;   // 
    //Port_TMRSTB_0;   //   Power On
    Port_SPI_SS_0;   // SPI
    SPDR = ch << 1;  //   (  . )
    SMCR = 0;        // Sleep
    OffEn = 0;       // ,    
    OnState = 1;     // 
  }
}

//--------------------------  : --------------------------

__monitor void Main_OffEn(void)
{
  if(!OffEn)
  {
    SampleCount = 0; //   ExtPower
    ExtPower = 0;    //  ExtPower
    OffEn = 1;       // 
  }
}

//------------------------------ : ---------------------------------

__monitor void Main_PowerOff(void)
{
  if(OnState)
  {
    RTC_ShowClock(); // 
    Port_TMROUT_0;   // 
    Port_SPI_SS_1;   // SPI,  TX  Hi-Z
    SMCR = (1<<SM1) | (1<<SM0) | (1<<SE); // Power Save
    Port_TMRSTB_1;   //   Power On
    OnState = 0;     // 
  }
}

//--------------------------  : -------------------------------

bool Main_PowerState(void)
{
  return(OnState);
}

//----------------------------------------------------------------------------
