//----------------------------------------------------------------------------

//RT-9510S LCD
//RTC support module

//----------------------------------------------------------------------------

//    Timer2    
//   32768 .      :
//32768 / 256 / 128 (prescaler) = 1.    ,
//       RC-
// 8 .      (, ,
//,  ),   LCD,    
// .     200 .

//----------------------------------------------------------------------------

#include "Main.h"
#include "RTC.h"
#include "Display.h"

//----------------------------- : -----------------------------------

// :
enum
{
  DISP_CLOCK,
  DISP_TIMER1,
  DISP_TIMER2,
  DISP_RADIO
};

#define OFF_EN_DELAY 2 //  

//----------------------------- : ----------------------------------

static char DispMode;         //  
static time Now;              // 
static char Chan0;            //  (dummy)
static time Time1;            //  1
static char Chan1;            //  1
static time Time2;            //  2
static char Chan2;            //  2
static time OffTime;          // 
static char TmrChan;          //  
static time *EditTime;        //    
static char *EditChan;        //    
static char OffEnTimer;       //  

//--------------------------  : ------------------------------

void RTC_InitTime(time *t); // 
bool RTC_IsTime(time *now, time *tmr); //   

//---------------------------  RTC: ----------------------------

void RTC_Init(void)
{
  ASSR = (1<<AS2); //select asynchronous operation of TCNT2
  TCNT2 = 0;       //clear TCNT2
  //select precaler: 32.768 kHz / 256 / 128 = 1 Hz:
  TCCR2A = (1<<CS22) | (1<<CS20);
  //wait for TCN2UB and TCR2UB to be cleared:
  while(ASSR & ((1<<TCR2UB) | (1<<TCN2UB)));
  //clear interrupt flags:
  TIFR2 = 0xFF;
  //enable Timer2 overflow interrupt:
  TIMSK2 = (1<<TOIE2);
  //initial time and day setting:
  RTC_InitTime(&Now);
 // :
  RTC_InitTime(&Time1);
  Chan1 = 0;
  RTC_InitTime(&Time2);
  Chan2 = 0;
  OffTime.day = 0;        //  
  RTC_ShowClock();        // :
  TmrChan = 0;            //  
  OffEnTimer = 0;         //   
}

//------------------------------  RTC: ----------------------------

#pragma vector = TIMER2_OVF_vect
__interrupt void TIMER2_OVF_interrupt(void)
{
  //Port_TMRSTB_1;   //CALIBRATION
  // :
  if(++Now.sec == 60)
  {
    Now.sec = 0;
    if(++Now.min == 60)
    {
      Now.min = 0;
      if(++Now.hrs == 24)
      {
        Now.hrs = 0;
        if(++Now.day == 8)
        {
          Now.day = 1;
	}
      }
    }
  }

  // :
  if(DispMode == DISP_CLOCK)
    LCD_ShowClock(&Now);

  // :
  if(OffEnTimer)
  {
    OffEnTimer--;
    if(!OffEnTimer) Main_OffEn();
  }

  // :
  if(Pin_POWON)
  {
    if(Main_PowerState() == 0)
    {
      Main_PowerOn(0);
      OffEnTimer = OFF_EN_DELAY;
    }
  }

  // :
  else
  {
    //  1 .   :
    if(TmrChan)
    {
      Main_PowerOn(TmrChan);
      OffEnTimer = OFF_EN_DELAY;
      TmrChan = 0;
    }
    //  1:
    if(Chan1 && RTC_IsTime(&Now, &Time1))
    {
      RTC_SetOffTime();
      Main_PowerOff();
      TmrChan = Chan1;
    }
    //  2:
    if(Chan2 && RTC_IsTime(&Now, &Time2))
    {
      RTC_SetOffTime();
      Main_PowerOff();
      TmrChan = Chan2;
    }
    //  :
    if(OffTime.day && RTC_IsTime(&Now, &OffTime))
    {
      Main_PowerOff();
      OffTime.day = 0;
    }
  }
  //Port_TMRSTB_0;   //CALIBRATION
}

//-------------------------  : ---------------------------

void RTC_InitTime(time *t)
{
  t->sec  = 0x00;
  t->min  = 0x01;
  t->hrs  = 0x01;
  t->day  = 0x01;
}

//-----------------------   : --------------------------

bool RTC_IsTime(time *now, time *tmr)
{
  if(!tmr->day) tmr->day = now->day;
  return((now->sec == tmr->sec) &&
         (now->min == tmr->min) &&
         (now->hrs == tmr->hrs) &&
         (now->day == tmr->day));
}

//----------    (OffTime = Now + 1hr): ------------

__monitor void RTC_SetOffTime(void)
{
  OffTime.sec = Now.sec;
  OffTime.min = Now.min;
  OffTime.hrs = Now.hrs;
  OffTime.day = Now.day;
  if(++OffTime.hrs == 24)
  {
    OffTime.hrs = 0;
    if(++OffTime.day == 8)
    {
      OffTime.day = 1;
    }
  }
}

//---------------------------------------------------------------------------
//-------------------------  : -----------------------------
//---------------------------------------------------------------------------

//---------------------   : -------------------------

__monitor void RTC_ShowClock(void)
{
  DispMode = DISP_CLOCK;
  EditTime = &Now;
  EditChan = &Chan0;
  LCD_ShowClock(&Now);
}

//--------------------    1: ------------------------

__monitor void RTC_ShowTimer1(void)
{
  DispMode = DISP_TIMER1;
  EditTime = &Time1;
  EditChan = &Chan1;
  LCD_ShowTimer(1, &Time1, Chan1);
}

//--------------------    2: ------------------------

__monitor void RTC_ShowTimer2(void)
{
  DispMode = DISP_TIMER2;
  EditTime = &Time2;
  EditChan = &Chan2;
  LCD_ShowTimer(2, &Time2, Chan2);
}

//----------------------   : --------------------------

void RTC_ShowRadio(char b, int f, char ch)
{
  DispMode = DISP_RADIO;
  LCD_ShowRadio(b, f, ch);
}

//-------------------    : -----------------------

__monitor void RTC_IncHrs(void)
{
  char h = EditTime->hrs;
  if(++h == 24) h = 0;
  EditTime->hrs = h;
  if(DispMode == DISP_CLOCK) LCD_ShowClock(EditTime);
    else LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, *EditChan);
}

//------------------    : ----------------------

__monitor void RTC_Inc10m(void)
{
  char m = EditTime->min;
  if(m >= 50) m = m % 10;
    else m = m + 10;
  EditTime->min = m;
  if(DispMode == DISP_CLOCK) LCD_ShowClock(EditTime);
    else LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, *EditChan);
}

//-------------------    : -----------------------

__monitor void RTC_IncMin(void)
{
  char m = EditTime->min;
  if(m % 10 == 9) m = m - 9;
    else m = m + 1;
  EditTime->min = m;
  if(DispMode == DISP_CLOCK) LCD_ShowClock(EditTime);
    else LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, *EditChan);
}

//----------------------   : --------------------------

__monitor void RTC_AdjSec(void)
{
  EditTime->sec = 0;
  char m = EditTime->min;
  if(m > 29)
  {
    if(++EditTime->min == 60)
    {
      EditTime->min = 0;
      if(++EditTime->hrs == 24)
      {
        EditTime->hrs = 0;
        if(++EditTime->day == 8)
        {
          EditTime->day = 1;
	}
      }
    }
  }
  LCD_ShowClock(EditTime);
}

//------------------------   : ----------------------------

__monitor void RTC_IncWeek(void)
{
  char m = EditTime->day;
  if(m == 7) m = ((DispMode == DISP_CLOCK)? 1 : 0);
    else m = m + 1;
  EditTime->day = m;
  if(DispMode == DISP_CLOCK) LCD_ShowClock(EditTime);
    else LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, *EditChan);
}

//-----------------    (0 - ): -----------------

__monitor void RTC_IncChan(void)
{
  char m = *EditChan;
  if(++m > CHANS) m = 0;
  *EditChan = m;
  LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, m);
}

//-------------------    (0 - ): -------------------

__monitor void RTC_Ch0(void)
{
  *EditChan = 0;
  LCD_ShowTimer((DispMode == DISP_TIMER1)? 1 : 2, EditTime, 0);
}

//---------------------------------------------------------------------------
